/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2012, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     encmain.cpp
    \brief    Encoder application main
*/

#include <time.h>
#include "TAppEncTop.h"

//! \ingroup TAppEncoder
//! \{

// ====================================================================================================================
// Main function
// ====================================================================================================================

int main(int argc, char* argv[])
{
  TAppEncTop  cTAppEncTop;

#ifdef HEVC_SVC
  std::string s1;
  std::string s2("-pf");
  std::string s3("-Ilp");
  std::string s4("-ilp");
  std::string main_param[MAX_CMD_LINE_PARA];
  char* pchBitstreamFile=NULL;
  std::string layerCfgFile[MAX_LAYERS];
  unsigned layerId = 0;
  unsigned maxLayerNum = 0;
  unsigned n, m;
  unsigned framerate = 30;
  unsigned GOP_length = 16;
  unsigned framesToBeEncoded = 128;
  Int      log2Factor = 1;
#endif

  // print information
  fprintf( stdout, "\n" );
  fprintf( stdout, "HM software: Encoder Version [%s]", NV_VERSION );
  fprintf( stdout, NVM_ONOS );
  fprintf( stdout, NVM_COMPILEDBY );
  fprintf( stdout, NVM_BITS );
  fprintf( stdout, "\n" );

  // create application encoder class
  cTAppEncTop.create();

#ifdef HEVC_SVC     // read main scalable configuration file '-pf'
  for (n = 1; n < argc; n++)
  {
      s1 = argv[n++];
      if(s1 == s2)
      {
          std::string cFilename = argv[n];
          FILE *f = fopen(cFilename.c_str(), "r");
          if ( NULL == f )
          {
              cout << "Failed to open cfg file " << cFilename.c_str() << endl;
              if (pchBitstreamFile!= NULL) free(pchBitstreamFile);
              return 0;
          }
          while (!feof(f))
          {
              for (m = 0; m < MAX_CMD_LINE_PARA; m ++)
                  main_param[m] = "";

              bool isComment = false;
              unsigned char uiTagNum = 0;

              for(m = 0; ; m++)
              {
                  char cChar = (char) fgetc(f);
                  if(cChar == '\n' || feof(f))
                      break;
                  if(cChar == '#')
                      isComment = true;

                  if(!isComment)
                  {
                      if (cChar == '\t' || cChar == ' ')
                      {
                          if (!main_param[uiTagNum].empty())
                              uiTagNum ++;
                      }
                      else
                      {
                          main_param[uiTagNum] += cChar;
                      }
                  }
              }
              if(main_param[0].empty() )
                  continue;
              if( main_param[0] == "OutputFile")
                  #ifdef _MSC_VER
                  pchBitstreamFile = main_param[1].empty()? NULL : _strdup(main_param[1].c_str());
                  #else
                  pchBitstreamFile = main_param[1].empty()? NULL : strdup(main_param[1].c_str());
                  #endif
              if( main_param[0] == "NumLayers")
                  maxLayerNum = (int)atof(main_param[1].c_str());
              if( main_param[0] == "LayerCfg")
                  layerCfgFile[layerId++] = main_param[1].c_str();
              if( main_param[0] == "FrameRate")
                  framerate = (unsigned)atof(main_param[1].c_str());
              if( main_param[0] == "FramesToBeEncoded")
                  framesToBeEncoded = (unsigned)atof(main_param[1].c_str());
              if( main_param[0] == "GOPSize")
                  GOP_length = (unsigned)atof(main_param[1].c_str());
          }  // foef
      }  // if "-pf"
      else if((s1 == s3) || (s1 == s4))  // ILP 
      {
      }
  }
#endif

#ifdef HEVC_SVC
  for (n = 0; n < maxLayerNum; n ++)
  {
      cTAppEncTop.setCurrentLayerCfg(n);
      cTAppEncTop.setFrameToBeEncoded( framesToBeEncoded );
      if(!cTAppEncTop.parseCfg( layerCfgFile[n], pchBitstreamFile ))
      {
        cTAppEncTop.destroy();
        return 1;
      }
 
      cTAppEncTop.setMaxTemporalLayers( cTAppEncTop.getLogFactor(1, GOP_length) + 1);
      cTAppEncTop.setGOPSize(GOP_length);
      log2Factor = cTAppEncTop.getLogFactor(cTAppEncTop.getFrameRate(), framerate);
      if (log2Factor < 0 )
      {
          fprintf( stderr, "\nMaximum frame rate must be a power of 2 of input frame rate\n");
          if (pchBitstreamFile!= NULL) free(pchBitstreamFile);
          return 1;
      }
      cTAppEncTop.setTemporalResolution ( (UInt)log2Factor );
      cTAppEncTop.determineIlpMode(); 
      cTAppEncTop.encodeInit( n );      // initialize the encoder 
      if (cTAppEncTop.getIlpMode()>0)
      {   
        cTAppEncTop.upSampleInit( n );    // initialize up sampling
      }
  } 
#else  // not HEVC_SVC
  // parse configuration
  if(!cTAppEncTop.parseCfg( argc, argv ))
  {
    cTAppEncTop.destroy();
    return 1;
  } 
#endif  // HEVC_SVC

  // starting time
  double dResult;
  long lBefore = clock();

#ifdef HEVC_SVC  
  fstream bitstreamFile(pchBitstreamFile, fstream::binary | fstream::out);

  if (!bitstreamFile)
  {
        fprintf(stderr, "\nfailed to open bitstream file `%s' for writing\n", pchBitstreamFile);
        exit(EXIT_FAILURE);
  } 
#endif

  // call encoding function
#ifdef HEVC_SVC
  cTAppEncTop.encode(maxLayerNum, &bitstreamFile);
#else
  cTAppEncTop.encode();
#endif

  // ending time
  dResult = (double)(clock()-lBefore) / CLOCKS_PER_SEC;
  printf("\n Total Time: %12.3f sec.\n", dResult);

  // destroy application encoder class
  cTAppEncTop.destroy();

#ifdef HEVC_SVC
  bitstreamFile.close();
  if (pchBitstreamFile != NULL) free(pchBitstreamFile);
#endif

  return 0;
}

//! \}
