/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TComUpSampleFilter.cpp
    \brief    up sample filter class
*/

//! \ingroup TLibCommon
//! \{

#include "TLibCommon/CommonDef.h"
#include "TLibCommon/TComUpSampleFilter.h"

// add_coef_set here ******************************
// make sure to update COEF_SET_e
#ifndef NEW_FIXED_FILTER
//const int m_nFilterPhases[MAX_NUMBER_COEF_SETS]= {3};     
const int m_nCoefPerPhase_h[MAX_NUMBER_COEF_SETS] = {11};   
const int m_nCoefPerPhase_v[MAX_NUMBER_COEF_SETS] = {11};   
int m_nShifts_h[MAX_NUMBER_COEF_SETS] = {11};
int m_nShifts_v[MAX_NUMBER_COEF_SETS] = {11};

/* 
TODO-cbal: Filter coefficients have to be defined altogether, 
no need to define different set for vertical and horizontal.
Then the appropriate phase will be selected within the FilterImg
function. For now, this is a quick hack!

Actual filters used:

1to2 - 8 phase filter
   [0,    0,    0,    0,    0, 2048,    0,    0,    0,    0,    0, // 0/8 - integer pel
    0,   28,  -51,   95, -208, 1979,  273, -113,   59,  -33,   17, // 1/8
    0,   47,  -87,  161, -339, 1813,  591, -226,  118,  -64,   35, // 2/8
    0,   57, -105,  193, -395, 1567,  930, -323,  165,  -90,   49, // 3/8
    0,   57, -105,  193, -385, 1264, 1264, -385,  193, -105,   57, // 4/8
    0,   49,  -90,  165, -323,  930, 1567, -395,  193, -105,   57, // 5/8
    0,   35,  -64,  118, -226,  591, 1813, -339,  161,  -87,   47, // 6/8
    0,   17,  -33,   59, -113,  273, 1979, -208,   95,  -51,   28] // 7/8

2to3 - 12 phase filter
   [0,    0,    0,    0,    0, 2048,    0,    0,    0,    0,    0, //  0/12 - integer pel
    0,   19,  -35,   66, -147, 2013,  176,  -74,   39,  -21,   12, //  1/12
    0,   35,  -65,  121, -260, 1934,  375, -151,   79,  -43,   23, //  2/12
    0,   47,  -87,  161, -339, 1813,  591, -226,  118,  -64,   35, //  3/12
    0,   55, -101,  186, -384, 1657,  816, -294,  151,  -83,   45, //  4/12
    0,   58, -107,  197, -398, 1471, 1043, -349,  177,  -97,   52, //  5/12
    0,   57, -105,  193, -385, 1264, 1264, -385,  193, -105,   57, //  6/12
    0,   52,  -97,  177, -349, 1043, 1471, -398,  197, -107,   58, //  7/12
    0,   45,  -83,  151, -294,  816, 1657, -384,  186, -101,   55, //  8/12
    0,   35,  -64,  118, -226,  591, 1813, -339,  161,  -87,   47, //  9/12
    0,   23,  -43,   79, -151,  375, 1934, -260,  121,  -65,   35, // 10/12
    0,   12,  -21,   39,  -74,  176, 2013, -147,   66,  -35,   19] // 11/12
*/

int m_fcoef_1to2_v[MAX_NUMBER_COEF_SETS][MAX_COEFS] = {
    {0,    0,    0,    0,    0, 2048,    0,    0,    0,    0,    0, // 0/8
     0,   57, -105,  193, -385, 1264, 1264, -385,  193, -105,   57} // 4/8
};
int m_fcoef_1to2_v_chr[MAX_NUMBER_COEF_SETS][MAX_COEFS] = {
    {0,   57, -105,  193, -395, 1567,  930, -323,  165,  -90,   49, // 3/8
     0,   17,  -33,   59, -113,  273, 1979, -208,   95,  -51,   28} // 7/8
};
int m_fcoef_1to2_h[MAX_NUMBER_COEF_SETS][MAX_COEFS] = { 
    {0,    0,    0,    0,    0, 2048,    0,    0,    0,    0,    0, // 0/8
     0,   57, -105,  193, -385, 1264, 1264, -385,  193, -105,   57} // 4/8
};
int m_fcoef_2to3_v[MAX_NUMBER_COEF_SETS][MAX_COEFS] = {
    {0,    0,    0,    0,    0, 2048,    0,    0,    0,    0,    0, //  0/12
     0,   55, -101,  186, -384, 1657,  816, -294,  151,  -83,   45, //  4/12
     0,   45,  -83,  151, -294,  816, 1657, -384,  186, -101,   55} //  8/12
};
int m_fcoef_2to3_v_chr[MAX_NUMBER_COEF_SETS][MAX_COEFS] = {
    {0,   47,  -87,  161, -339, 1813,  591, -226,  118,  -64,   35, //  3/12
     0,   52,  -97,  177, -349, 1043, 1471, -398,  197, -107,   58, //  7/12
     0,   12,  -21,   39,  -74,  176, 2013, -147,   66,  -35,   19} // 11/12
};
int m_fcoef_2to3_h[MAX_NUMBER_COEF_SETS][MAX_COEFS] = { 
    {0,    0,    0,    0,    0, 2048,    0,    0,    0,    0,    0, //  0/12
     0,   55, -101,  186, -384, 1657,  816, -294,  151,  -83,   45, //  4/12
     0,   45,  -83,  151, -294,  816, 1657, -384,  186, -101,   55} //  8/12
};
// end add_coef_set here ******************************
#else

const int m_nCoefPerPhase[MAX_NUMBER_COEF_SETS] = {9};   
const int m_nCoefPerPhase_chr[MAX_NUMBER_COEF_SETS] = {5};   
int m_nShifts[MAX_NUMBER_COEF_SETS] = {7};

int m_fcoef_1to2[MAX_NUMBER_COEF_SETS][MAX_COEFS] = 
{   
   {0,     0,     0,     0,   128,     0,     0,     0,     0, // 0/8
    0,    -3,     8,   -21,    80,    80,   -21,     8,    -3} // 4/8
}; 
int m_fcoef_1to2_v_chr[MAX_NUMBER_COEF_SETS][MAX_COEFS] = 
{
    {0,   -17,   100,    56,   -11,  // 3/8
   0,    -3,    15,   126,   -10}  // 7/8
};
int m_fcoef_1to2_h_chr[MAX_NUMBER_COEF_SETS][MAX_COEFS] =  
{
  {0,     0,   128,     0,     0, // 0/8
   0,   -15,    79,    79,   -15} // 4/8
};
int m_fcoef_2to3[MAX_NUMBER_COEF_SETS][MAX_COEFS] =  
{
    {0,     0,     0,     0,   128,     0,     0,     0,     0, // 0/12 
     0,    -5,    11,   -24,   107,    52,   -18,     8,    -4, // 4/12 
   0,    -4,     8,   -18,    52,   107,   -24,    11,    -5} // 8/12  
};
int m_fcoef_2to3_v_chr[MAX_NUMBER_COEF_SETS][MAX_COEFS] =  
{
    {0,   -20,   122,    38,   -12,  // 3/12 
     0,   -20,    71,   101,   -24,  // 7/12  
   0,    -4,    12,   128,    -8} // 11/12  
};
int m_fcoef_2to3_h_chr[MAX_NUMBER_COEF_SETS][MAX_COEFS] =  
{
    {0,     0,   128,     0,     0, // 0/12 
     0,   -23,   113,    54,   -16, // 4/12 
     0,   -16,    54,   113,   -23} // 8/12 
};
#endif

COEF_SET_e m_coefSetX;

// ====================================================================================================================
/**
 * \brief           selects the filter coefficient set
*/
// ====================================================================================================================
void TComUpSampleFilter:: selectFilterCoefSet(COEF_SET_e setX)
{
    m_coefSetX = setX;
}

// ====================================================================================================================
/**
 * \brief           sets up sampling ratio
*/
// ====================================================================================================================
void TComUpSampleFilter::  setUpSampleRatio(upSampleRatio_e r)  
{
    m_upSampleRatio = r;
    if (r == ONE_TO_ONE)
    {
        m_pictueWidthUpSampled = m_pictueWidthOriginal;
        m_pictueHeightUpSampled = m_pictueHeightOriginal;
    }
#ifndef NEW_FIXED_FILTER
    else if (r == ONE_TO_TWO)
    {
        m_pictueWidthUpSampled = 2 * m_pictueWidthOriginal;
        m_pictueHeightUpSampled = 2 * m_pictueHeightOriginal;
        m_fcoef_v     = m_fcoef_1to2_v    [m_coefSetX];
        m_fcoef_v_chr = m_fcoef_1to2_v_chr[m_coefSetX];
        m_fcoef_h     = m_fcoef_1to2_h    [m_coefSetX];
    }
    else
    {
        m_pictueWidthUpSampled = 3 * m_pictueWidthOriginal/2;
        m_pictueHeightUpSampled = 3 * m_pictueHeightOriginal/2;
        m_fcoef_v     = m_fcoef_2to3_v    [m_coefSetX];
        m_fcoef_v_chr = m_fcoef_2to3_v_chr[m_coefSetX];
        m_fcoef_h     = m_fcoef_2to3_h    [m_coefSetX];
    }
#else
    else if (r == ONE_TO_TWO)
    {
        m_pictueWidthUpSampled = 2 * m_pictueWidthOriginal;
        m_pictueHeightUpSampled = 2 * m_pictueHeightOriginal;
        m_fcoef_v     = m_fcoef_1to2      [m_coefSetX];
        m_fcoef_v_chr = m_fcoef_1to2_v_chr[m_coefSetX];
        m_fcoef_h     = m_fcoef_1to2      [m_coefSetX];
        m_fcoef_h_chr = m_fcoef_1to2_h_chr[m_coefSetX];
    }
    else
    {
        m_pictueWidthUpSampled = 3 * m_pictueWidthOriginal/2;
        m_pictueHeightUpSampled = 3 * m_pictueHeightOriginal/2;
        m_fcoef_v     = m_fcoef_2to3      [m_coefSetX];
        m_fcoef_v_chr = m_fcoef_2to3_v_chr[m_coefSetX];
        m_fcoef_h     = m_fcoef_2to3       [m_coefSetX];
        m_fcoef_h_chr = m_fcoef_2to3_h_chr[m_coefSetX];
    }
#endif

}

// ====================================================================================================================
/**
 * \brief         The up sample filter\n
 *                Does horizontal filtering then vertical filtering
 *
 * \param[in]     *src //data to be filtered
 * \param[in]     *temp - output of row filter
 * \param[out]    *dst  - output (filtered data)
 * \param[in]     height1  - original height of picture
 * \param[in]     width1 - original width of picture
 * \param[in]     Mh - up sample by M/N ratio
 * \param[in]     Nh 
 * \param[in]     Mv 
 * \param[in]     Nv 
 * \param[in]     phase_filter_h[2][11] - row filter coefficients
 * \param[in]     phase_filter_v[2][11] - column filter coefficients
 * \param[in]     length_h - number of taps in the row filter
 * \param[in]     length_v - number of taps in the column filter
 * \param[in]     shift_h  - for scaling
 * \param[in]     shift_v)
*/
// ====================================================================================================================
void TComUpSampleFilter:: FilterImg(
        Pel           *src,                         //data to be filtered
        int           *temp,                        // output of row filter
        Pel           *dst,                         // output (filtered data)
        int           height1,                      // original height of picture
        int           width1,                       // original width of picture  
        int           originalRowWidth,             // input row step
        int           upSampledRowWidth,            // output row step
        int           Mh,                           // up sample by M/N ratio
        int           Nh, 
        int           Mv, 
        int           Nv, 
        int           phase_filter_h[],             // row filter coefficients
        int           phase_filter_v[],             // column filter coefficients
        int           length,                     // number of taps in the row filter
        int           shift,                      // for scaling
        int           is_chroma)
{
    int height2,width2;
    int k,iSum;
    int i0,i1,i2,i_phase;
    int j0,j1,j2,j_phase;
    int currX, startX, coefX, padX = (length/2);
    int currY, lastY,  coefY, padY = (length/2); 

    height2 = height1*Mv/Nv;  // up sampled height
    width2  = width1 *Mh/Nh;  // up sampled width

    // horizontal filtering
    for(j1=0;j1<height1;j1++) // for each row (original height)
    {
        startX = j1 * originalRowWidth;
        i0=-Nh;
        for(i2=0;i2<width2;i2++)  // for each column
        {
            i0     += Nh;
            i1      = i0/Mh;
            i_phase = i0%Mh;
            iSum=0;       
            for(k=0;k<(length+1);k++) // for each filter coefficient
            {
                currX = i1 - padX + k;
                coefX = i_phase * (length+1) + k;
                if (currX < 0)            // left margin
                    iSum += src[startX             ]*phase_filter_h[coefX];
                else if (currX < width1)  // middle
                    iSum += src[startX + currX     ]*phase_filter_h[coefX];
                else                      // right margin
                    iSum += src[startX + width1 - 1]*phase_filter_h[coefX];
            }
            // Example, for CIF, temp is 704x586 = 2*352 x (2*288 + 10)
            temp[j1 * width2 + i2]=iSum;
        }
    }

    // vertical filtering
    lastY = (height1-1) * width2;  // beginning address of the last row available in temp 
    is_chroma == 0 ? j0=-Nv : j0=-Nv-1; // phase adjustment due to chroma sampling pattern wrt to luma samples
    for(j2=0;j2<height2;j2++)  // for each row (up sampled height)
    {
        j0     += Nv;
        j1      = j0/Mv;
        j_phase = j0%Mv;
        if( j0<0 && j_phase<0 ) // when j0 is negative and not a multiple of Mv
        {
            j1-=1;              // round down to find the correct integer-pel
            j_phase+=Mv;        // calculate the positive modulus 
        }

        for(i2=0;i2<width2;i2++)  // for each column
        {
            iSum=0;
            for(k=0;k<(length+1);k++)  // for each filter coefficient
            {
                currY = j1 - padY + k;
                coefY = j_phase * (length+1) + k;
                if (currY < 0)            // top margin
                    iSum += temp[i2                 ]*phase_filter_v[coefY];
                else if (currY < height1) // middle
                    iSum += temp[i2 + currY * width2]*phase_filter_v[coefY];
                else                      // bottom margin
                    iSum += temp[i2 + lastY         ]*phase_filter_v[coefY];
            }
            iSum = ( ( iSum + ( 1 << (shift+shift-1) ) ) >> (shift+shift) );
            dst[j2*upSampledRowWidth+i2] = (unsigned char)(iSum>255 ? 255 : iSum<0 ? 0 : iSum);
        }
    }
}

// ====================================================================================================================
/**
 * \brief           constructor for the up sample filter
*/
// ====================================================================================================================
TComUpSampleFilter::TComUpSampleFilter() // constructor
{ 
    m_upSampleId = -1;  // for debug
    m_pictueWidthOriginal = 0;
    m_pictueHeightOriginal = 0;
    m_pictueWidthUpSampled = 0;
    m_pictueHeightUpSampled = 0;
    m_upSampleRatio = USR_NOT_ASSIGNED;
    m_temp = NULL;
    m_coefSetX = COEF_SET_1;
}  

// ====================================================================================================================
/**
 * \brief           copies data from in to out \n
 *                  Input and output format EXAMPLE \n
 *                  352 x 288 is stored in a 512 x 432 array\n
 *                  Columns             \n
 *                  lumaMaginX  Data    lumaMaginX  Total\n
 *                  80          352     80          512 shorts\n
 *\n
 *                  72  lumaMaginY rows\n
 *                  288 data rows\n
 *                  72  lumaMaginY rows\n
 *                  432 total rows\n
 *\n
 *                  1024    bytes per row\n
 *                  442368  bytes total\n
 *\n
 * \param[in]       in the input array
 * \param[out]      out the output array
 * \param[in]       nRows the number of rows
 * \param[in]       nCols the number of columns
 * \param[in]       fullRowWidth the row width which includes the Luma/Chroma margin
**/
// ====================================================================================================================
void copyOnetoOne(
                  Pel *in,        
                  Pel *out,      
                  int nCols,
                  int nRows, 
                  int fullRowWidth)
{
    int rX, cX;

    for (rX = 0; rX < nRows; rX++)       
    {
        for (cX = 0; cX < nCols; cX++) 
        {
            out[cX] = in[cX];
        }
        in = in + fullRowWidth;
        out = out + fullRowWidth;

    }
}


void compareOnetoOne(
                  int num,
                  Pel *in1,        
                  Pel *in2,      
                  int nCols,
                  int nRows, 
                  int fullRowWidth)
{
    int rX, cX;

    for (rX = 0; rX < nRows; rX++)       
    {
        for (cX = 0; cX < nCols; cX++) 
        {
            if (in1[cX] != in2[cX])
            {
                printf("mismatch num %d rx %d cx %d values %d %d\n",num,rX, cX,in1[cX] , in2[cX] );
                exit(0);
            }
        }
        in1 = in1 + fullRowWidth;
        in2 = in2 + fullRowWidth;

    }
}

void TComUpSampleFilter:: compareUpsampledPictureYuv(TComPicYuv*   pcPicYuvIn1, TComPicYuv*   pcPicYuvIn2)
{
   
    int usampledRowWidthLuma = 2 * pcPicYuvIn1->getLumaMargin() + m_pictueWidthUpSampled; 
    int usampledRowWidthCroma = 2 * pcPicYuvIn1->getChromaMargin() + (m_pictueWidthUpSampled>>1);

    //printf("compareUpsampledPictureYuv ID %d  %dx%d  to  %dx%d\n",m_upSampleId,m_pictueWidthOriginal,m_pictueHeightOriginal,m_pictueWidthUpSampled,m_pictueHeightUpSampled);

    compareOnetoOne(
        1,
        pcPicYuvIn1->getLumaAddr(),        
        pcPicYuvIn2->getLumaAddr(),      
        m_pictueWidthUpSampled, 
        m_pictueHeightUpSampled,
        usampledRowWidthLuma);
    compareOnetoOne(
        2,
        pcPicYuvIn1->getCrAddr(),        
        pcPicYuvIn2->getCrAddr(),      
        m_pictueWidthUpSampled/2, 
        m_pictueHeightUpSampled/2,
        usampledRowWidthCroma);
    compareOnetoOne(
        3,
        pcPicYuvIn1->getCbAddr(),        
        pcPicYuvIn2->getCbAddr(),      
        m_pictueWidthUpSampled/2, 
        m_pictueHeightUpSampled/2,
        usampledRowWidthCroma);
}

void TComUpSampleFilter:: copyUpsampledPictureYuv_notUsed(TComPicYuv*   pcPicYuvIn, TComPicYuv*   pcPicYuvOut)
{
    int upsampledRowWidthLuma = 2 * pcPicYuvIn->getLumaMargin() + m_pictueWidthUpSampled; 
    int upsampledRowWidthCroma = 2 * pcPicYuvIn->getChromaMargin() + (m_pictueWidthUpSampled>>1);

    copyOnetoOne(
        pcPicYuvIn->getLumaAddr(),        
        pcPicYuvOut->getLumaAddr(),      
        m_pictueWidthUpSampled, 
        m_pictueHeightUpSampled,
        upsampledRowWidthLuma);
    copyOnetoOne(
        pcPicYuvIn->getCrAddr(),        
        pcPicYuvOut->getCrAddr(),      
        m_pictueWidthUpSampled/2, 
        m_pictueHeightUpSampled/2,
        upsampledRowWidthCroma);
    copyOnetoOne(
        pcPicYuvIn->getCbAddr(),        
        pcPicYuvOut->getCbAddr(),      
        m_pictueWidthUpSampled/2, 
        m_pictueHeightUpSampled/2,
        upsampledRowWidthCroma);
}

// ====================================================================================================================
/**
 * \brief           up samples a picture\n
 * \n
 *                  does the followig for YUV
 *                  copies picture into format expected by filter\n
 *                  applies the filter\n
 *                  copies data back to original format
 * \param[in]       pcPicIn contains the Picture to be up sampled     
 * \param[out]      pcPicOut contains the upsampled
*/
// ====================================================================================================================
void TComUpSampleFilter:: doTheUpSample(TComPic*   pcPicIn, TComPicYuv*   pcPicYuvOut, int testMode)
{
#ifndef NEW_FIXED_FILTER
    int nCoef = m_nCoefPerPhase_h[m_coefSetX]&0x7e;
    int Mh = 2, Mv = 2, Nh=1, Nv = 1;
    int shift_h=m_nShifts_h[m_coefSetX];
    int shift_v=m_nShifts_v[m_coefSetX];
#else
    int Mh = 2, Mv = 2, Nh=1, Nv = 1;
    int nCoef     = m_nCoefPerPhase[m_coefSetX]&0x7e;
    int nCoef_chr = m_nCoefPerPhase_chr[m_coefSetX]&0x7e;
    int shift     = m_nShifts[m_coefSetX];
#endif
    if (m_upSampleRatio  == ONE_TO_TWO)
    {
        Mh = 2, Mv = 2, Nh=1, Nv = 1;
    }
    else if (m_upSampleRatio  == TWO_TO_THREE)
    {
        Mh = 3, Mv = 3, Nh=2, Nv = 2;
    }
    else if (m_upSampleRatio  == ONE_TO_ONE)
    {
    }
    else
    {
        assert(!"Illegal up sample ratio");
    }
    int originalRowWidthLuma = 2 * pcPicIn->getPicYuvRec()->getLumaMargin() + m_pictueWidthOriginal; 
    int originalRowWidthCroma = 2 * pcPicIn->getPicYuvRec()->getChromaMargin() + (m_pictueWidthOriginal>>1);
    int upSampledRowWidthLuma = 2 * pcPicIn->getPicYuvRec()->getLumaMargin() +  m_pictueWidthUpSampled; 
    int upSampledRowWidthCroma = 2 * pcPicIn->getPicYuvRec()->getChromaMargin() +  (m_pictueWidthUpSampled>>1);    

     if (m_temp == NULL)
    {
        m_temp = (int *)  malloc(sizeof(int) * m_pictueWidthUpSampled * m_pictueHeightOriginal);
    }

    // Luma ********************
    if (m_upSampleRatio  == ONE_TO_ONE)
    {
        copyOnetoOne(
            pcPicIn->getPicYuvRec()->getLumaAddr(),        
            pcPicYuvOut->getLumaAddr(),      
            m_pictueWidthUpSampled, 
            m_pictueHeightUpSampled,
            originalRowWidthLuma);
        copyOnetoOne(
            pcPicIn->getPicYuvRec()->getCrAddr(),        
            pcPicYuvOut->getCrAddr(),      
            m_pictueWidthUpSampled/2, 
            m_pictueHeightUpSampled/2,
            originalRowWidthCroma);
        copyOnetoOne(
            pcPicIn->getPicYuvRec()->getCbAddr(),        
            pcPicYuvOut->getCbAddr(),      
            m_pictueWidthUpSampled/2, 
            m_pictueHeightUpSampled/2,
            originalRowWidthCroma);
    }
    else
    {
        Pel * filterOut = pcPicYuvOut->getLumaAddr();

#ifndef NEW_FIXED_FILTER
        FilterImg(
                   pcPicIn->getPicYuvRec()->getLumaAddr(),  //data to be filtered
                   m_temp,                      // output of row filter
                   filterOut,                   // output (filtered data)
                   m_pictueHeightOriginal,      // height of picture
                   m_pictueWidthOriginal,       // width of picture  
                   originalRowWidthLuma,        // input row step
                   upSampledRowWidthLuma,       // output row step
                   Mh,                          // up sample by M/N ratio
                   Nh, 
                   Mv, 
                   Nv, 
                   m_fcoef_h,                   // row filter coefficients
                   m_fcoef_v,                   // column filter coefficients
                   nCoef,                       // number of taps in the row filter
                   nCoef,                       // number of taps in the column filter
                   shift_h,                     // for scaling
                   shift_v,
                   0);
#else
        FilterImg(
                   pcPicIn->getPicYuvRec()->getLumaAddr(),  //data to be filtered
                   m_temp,                      // output of row filter
                   filterOut,                   // output (filtered data)
                   m_pictueHeightOriginal,      // height of picture
                   m_pictueWidthOriginal,       // width of picture  
                   originalRowWidthLuma,        // input row step
                   upSampledRowWidthLuma,       // output row step
                   Mh,                          // up sample by M/N ratio
                   Nh, 
                   Mv, 
                   Nv, 
                   m_fcoef_h,                   // row filter coefficients
                   m_fcoef_v,                   // column filter coefficients
                   nCoef,                       // number of taps in the row filter
                   shift,                     // for scaling
                   0);
#endif

        // Cr  ********************
        filterOut = pcPicYuvOut->getCrAddr();

#ifndef NEW_FIXED_FILTER
        FilterImg(
                   pcPicIn->getPicYuvRec()->getCrAddr(),             //data to be filtered
                   m_temp,    // output of row filter
                   filterOut,     // output (filtered data)
                   m_pictueHeightOriginal>>1,   // height of picture
                   m_pictueWidthOriginal>>1,    // width of picture
                   originalRowWidthCroma,
                   upSampledRowWidthCroma,
                   Mh,        // up sample by M/N ratio
                   Nh, 
                   Mv, 
                   Nv, 
                   m_fcoef_h,  // row filter coefficients
                   m_fcoef_v_chr,  // column filter coefficients
                   nCoef,              // number of taps in the row filter
                   nCoef,                 // number of taps in the column filter
                   shift_h,               // for scaling
                   shift_v,
                   1);
#else
        FilterImg(
                   pcPicIn->getPicYuvRec()->getCrAddr(),             //data to be filtered
                   m_temp,    // output of row filter
                   filterOut,     // output (filtered data)
                   m_pictueHeightOriginal>>1,   // height of picture
                   m_pictueWidthOriginal>>1,    // width of picture
                   originalRowWidthCroma,
                   upSampledRowWidthCroma,
                   Mh,        // up sample by M/N ratio
                   Nh, 
                   Mv, 
                   Nv, 
                   m_fcoef_h_chr,  // row filter coefficients
                   m_fcoef_v_chr,  // column filter coefficients
                   nCoef_chr,              // number of taps in the row filter
                   shift,               // for scaling
                   1);
#endif

        // Cb  ********************
        filterOut = pcPicYuvOut->getCbAddr();

#ifndef NEW_FIXED_FILTER
        FilterImg(
                   pcPicIn->getPicYuvRec()->getCbAddr(),             //data to be filtered
                   m_temp,    // output of row filter
                   filterOut,     // output (filtered data)
                   m_pictueHeightOriginal>>1,   // height of picture
                   m_pictueWidthOriginal>>1,    // width of picture 
                   originalRowWidthCroma,
                   upSampledRowWidthCroma,
                   Mh,        // up sample by M/N ratio
                   Nh, 
                   Mv, 
                   Nv, 
                   m_fcoef_h,  // row filter coefficients
                   m_fcoef_v_chr,  // column filter coefficients
                   nCoef,              // number of taps in the row filter
                   nCoef,                 // number of taps in the column filter
                   shift_h,               // for scaling
                   shift_v,
                   1);
#else
        FilterImg(
                   pcPicIn->getPicYuvRec()->getCbAddr(),             //data to be filtered
                   m_temp,    // output of row filter
                   filterOut,     // output (filtered data)
                   m_pictueHeightOriginal>>1,   // height of picture
                   m_pictueWidthOriginal>>1,    // width of picture 
                   originalRowWidthCroma,
                   upSampledRowWidthCroma,
                   Mh,        // up sample by M/N ratio
                   Nh, 
                   Mv, 
                   Nv, 
                   m_fcoef_h_chr,  // row filter coefficients
                   m_fcoef_v_chr,  // column filter coefficients
                   nCoef_chr,              // number of taps in the row filter
                   shift,               // for scaling
                   1);
#endif
    }

}

// ====================================================================================================================
/**
 * \brief           Currently not used.\n
 *                  For debugging.  Copies the data from m_apcPicYuv[1] to m_apcPicYuvUpsampledDebug
 *
 * \param[in,out]   pcPic - the picture
*/
// ====================================================================================================================
void TComUpSampleFilter:: upSample_1to1(TComPic*&   pcPic)
{
}

// ====================================================================================================================
/**
 * \brief           destructor for the up sample filter
*/
// ====================================================================================================================
TComUpSampleFilter::~TComUpSampleFilter()
{ 
    if (m_temp != NULL)
    {
        free(m_temp);
    }  

} 



