/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TComUpSampleFilter.h
    \brief    up sample filter class (header)
*/

#ifndef __TCOMUPSAMPLEILTER__
#define __TCOMUPSAMPLEILTER__

// Include files
#include <assert.h>
#include "TLibCommon/TComPic.h"

//! \ingroup TLibCommon
//! \{

// ====================================================================================================================
// Constants
// ====================================================================================================================
enum upSampleRatio_e
{
  ONE_TO_ONE = 0,
  ONE_TO_TWO,
  TWO_TO_THREE,
  USR_NOT_ASSIGNED
};
enum COEF_SET_e
{
  COEF_SET_1 = 0,
  COEF_SET_2,    // not used yet
  COEF_SET_3,    // not used yet
  MAX_NUMBER_COEF_SETS
};
#define MAX_COEFS 500

// Local type definitions

// ====================================================================================================================
// Class definition
// ====================================================================================================================

/// interpolation filter class
class TComUpSampleFilter { 
public: 
 
  TComUpSampleFilter();                // constructor 
 ~TComUpSampleFilter();       // destructor 
  
  void setPicutreWidthForUpsample(int w) 
  {
      m_pictueWidthOriginal = w;
  }

  void selectFilterCoefSet(COEF_SET_e setX) ;

  void setUpsampleId(int id) 
  {
      m_upSampleId = id;
  }
  int getUpsampleId() 
  {
      return m_upSampleId;
  }
  void setPicutreHeightForUpsample(int h) 
  {
      m_pictueHeightOriginal = h;
  }
  void setUpSampleRatio(upSampleRatio_e r);

  int getUpsampledWidth() {return m_pictueWidthUpSampled;}

  int getUpsampledHeight() {return m_pictueHeightUpSampled;}

  void setCurentlayerUpSample(int l)
  {
  }
  void upSample_1to1(TComPic*&   pcPic);    // for debug - takes in a picture and outouts the same picture - makes sure that the infra structure is in place

  void compareUpsampledPictureYuv(TComPicYuv*   pcPicYuvIn1, TComPicYuv*   pcPicYuvIn2);

  void copyUpsampledPictureYuv_notUsed(TComPicYuv*   pcPicYuvIn, TComPicYuv*   pcPicYuvOut);

  void doTheUpSample(TComPic*   pcPicIn, TComPicYuv*   pcPicOut,  int testMode);

private:
  int m_upSampleId; 
  int m_pictueWidthOriginal; 
  int m_pictueHeightOriginal;
  int m_pictueWidthUpSampled; 
  int m_pictueHeightUpSampled;
  upSampleRatio_e m_upSampleRatio;
   int *m_temp;

  int *m_fcoef_v;
  int *m_fcoef_v_chr;
  int *m_fcoef_h;
#ifdef NEW_FIXED_FILTER
  int *m_fcoef_h_chr;
#endif

#ifndef NEW_FIXED_FILTER
  void FilterImg(
               Pel           *src,                  //data to be filtered
               int           *temp,                 // output of row filter
               Pel           *dst,                  // output (filtered data)
               int           height1,               // height of picture
               int           width1,                // width of picture  
               int           originalRowWidth,      // input row step
               int           upSampledRowWidth,     // output row step
               int           Mh,                    // up sample by M/N ratio
               int           Nh, 
               int           Mv, 
               int           Nv, 
               int           phase_filter_h[],      // row filter coefficients
               int           phase_filter_v[],      // column filter coefficients
               int           length_h,              // number of taps in the row filter
               int           length_v,              // number of taps in the column filter
               int           shift_h,               // for scaling
               int           shift_v,
               int           is_chroma);
#else
  void FilterImg(
               Pel           *src,                  //data to be filtered
               int           *temp,                 // output of row filter
               Pel           *dst,                  // output (filtered data)
               int           height1,               // height of picture
               int           width1,                // width of picture  
               int           originalRowWidth,      // input row step
               int           upSampledRowWidth,     // output row step
               int           Mh,                    // up sample by M/N ratio
               int           Nh, 
               int           Mv, 
               int           Nv, 
               int           phase_filter_h[],      // row filter coefficients
               int           phase_filter_v[],      // column filter coefficients
               int           length,              // number of taps in the row filter
               int           shift,               // for scaling
               int           is_chroma);
#endif

};   

#endif // __TCOMPREDFILTER__
